//------------------------------------------------------------------------------
//	$Id: grid_gk.js,v 1.7 2019/06/25 13:35:28 wolf Exp $
//------------------------------------------------------------------------------
//	Erklaerung:	https://www.netzwolf.info/ol2/gkgrid.html
//------------------------------------------------------------------------------
//	Fragen, Wuensche, Bedenken, Anregungen?
//	<openlayers(%40)netzwolf.info>
//------------------------------------------------------------------------------

/*jsl:option explicit*/

'use strict';

OpenLayers.Layer.GridGK = OpenLayers.Class (OpenLayers.Layer.Vector, {

	initialize: function (name, options){

		OpenLayers.Layer.Vector.prototype.initialize.apply(this, [name, options]);
	},

	gridSizeText: null,

	gridSizeDiv: null,

	//----------------------------------------------------------------------
	//	Find matching grid unit or return null
	//----------------------------------------------------------------------

	getGridUnit: function (distance) {

		for (var i=0; i<this.gridUnits.length; i++) {

			if (distance<this.gridUnits[i]) { return this.gridUnits[i]; }
		}

		return null;
	},

	gridUnits: [50, 100, 200, 500, 1000, 2000, 5000, 10000, 20000],

	gridPixelDistance: 100,

	//----------------------------------------------------------------------
	//	Format gridsize
	//----------------------------------------------------------------------

	formatGridSize: function (m) {

		if (m>=1000 && m%1000===0) { return (m/1000)+"km"; }
		return m+"m";
	},

	//----------------------------------------------------------------------
	//	Set graphic attributes for grid
	//----------------------------------------------------------------------

	gridColor:	'blue',
	gridWidth:	function (value) { return value%1000===0? 2 : 1; },
	gridOpacity:	0.8,
	gridDashStyle:	'solid', // dot, dash, dashdot, longdash, longdashdot

	//----------------------------------------------------------------------
	//	Set graphic attributes for grid labels
	//----------------------------------------------------------------------

	gridLabel:	function (value) { return new String(value%1000===0? value/1000 : value); },
	gridLabelColor:	'black',
	gridLabelSize:	function (value) { return value%1000===0? '32px' : '12px'; },
	gridLabelAlign:	function (isHorizontal) { return isHorizontal? 'lb' : 'lt'; },

	//----------------------------------------------------------------------
	//	Draw grid on move or zoom
	//----------------------------------------------------------------------

	moveTo: function (bounds, zoomChanged, dragging) {

		//--------------------------------------------------------------
		//	but not while dragging
		//--------------------------------------------------------------

		if (dragging) { return; }

		//--------------------------------------------------------------
		//	Remove old grid
		//--------------------------------------------------------------

		this.destroyFeatures();

		//--------------------------------------------------------------
		//	Transform center and border to geogr. Coordinates
		//--------------------------------------------------------------

		var mapCenter = this.map.getCenter().clone().
			transform(this.map.getProjectionObject(), this.map.displayProjection);

		var mapBounds = bounds.clone().
			transform(this.map.getProjectionObject(), this.map.displayProjection);

		//--------------------------------------------------------------
		//	Number of GK-Zones in viewport
		//--------------------------------------------------------------

		var rZone = this.coarseLonToZone(mapBounds.right);
		var lZone = this.coarseLonToZone(mapBounds.left );

		var nZones= (rZone - lZone + 120) % 120 + 1;

		//--------------------------------------------------------------
		//	Set zone based on center of map
		//--------------------------------------------------------------

		if (nZones >= 3) {

			this.zone = null;

		} else if (nZones==1 || !this.zone || this.zone != lZone && this.zone != rZone) {

			this.zone = this.coarseLonToZone (mapCenter.lon);
		}

		//--------------------------------------------------------------
		//	GK-Coordinates of corners
		//--------------------------------------------------------------

		var ol = this.lonLatToGK (new OpenLayers.LonLat (mapBounds.left , mapBounds.top   ));
		var or = this.lonLatToGK (new OpenLayers.LonLat (mapBounds.right, mapBounds.top   ));
		var ul = this.lonLatToGK (new OpenLayers.LonLat (mapBounds.left , mapBounds.bottom));
		var ur = this.lonLatToGK (new OpenLayers.LonLat (mapBounds.right, mapBounds.bottom));

		//--------------------------------------------------------------
		//	Grid unit
		//--------------------------------------------------------------

		var unit = ur && ul ?
			this.getGridUnit ((ur.x-ul.x) / this.map.getSize().w * this.gridPixelDistance) : null;

		//--------------------------------------------------------------
		//	Grid size display object
		//	(TODO: create a OpenLayers.Control-Object)
		//--------------------------------------------------------------

		if (this.gridSizeText && !this.gridSizeDiv) {

			this.gridSizeDiv=OpenLayers.Util.createDiv(this.id);
			this.gridSizeDiv.className='olControlGridGK';
			this.gridSizeDiv.style.zIndex=this.map.Z_INDEX_BASE['Control']+ this.map.controls.length;
			this.gridSizeDiv.setAttribute("unselectable","on");
			this.map.viewPortDiv.appendChild (this.gridSizeDiv);
		}

		//--------------------------------------------------------------
		//	Hide grid size (if configured)
		//--------------------------------------------------------------

		if (this.gridSizeDiv) { this.gridSizeDiv.style.display='none'; }

		//--------------------------------------------------------------
		//	Create new grid
		//--------------------------------------------------------------

		if (this.zone && unit) {

			//------------------------------------------------------
			//	Compute grid
			//------------------------------------------------------

			var x1 = Math.floor (Math.min(ol.x, ul.x) / unit) * unit;
			var x2 = Math.ceil  (Math.max(or.x, ur.x) / unit) * unit;
			var y1 = Math.floor (Math.min(ul.y, ur.y) / unit) * unit;
			var y2 = Math.ceil  (Math.max(ol.y, or.y) / unit) * unit;

			var features = [];

			//------------------------------------------------------
			//	Vertical lines
			//------------------------------------------------------

			for (var x=x1; x<=x2; x+= unit) {

				var xu = (x - ul.x) / (ur.x-ul.x) * (bounds.right-bounds.left) +bounds.left;
				var xo = (x - ol.x) / (or.x-ol.x) * (bounds.right-bounds.left) +bounds.left;

				var v1 = new OpenLayers.Feature.Vector (new OpenLayers.Geometry.LineString([
					new OpenLayers.Geometry.Point (xo, bounds.top   ),
					new OpenLayers.Geometry.Point (xu, bounds.bottom)
				]));

				v1.style=this.gridStyle((x + 500000) % 10000000, false);

				features.push (v1);
			}

			//------------------------------------------------------
			//	Horizontal lines
			//------------------------------------------------------

			for (var y=y1; y<=y2; y+=unit) {

				var yl = (y - ul.y) / (ol.y-ul.y) * (bounds.top-bounds.bottom) +bounds.bottom;
				var yr = (y - ur.y) / (or.y-ur.y) * (bounds.top-bounds.bottom) +bounds.bottom;

				var v2 = new OpenLayers.Feature.Vector (new OpenLayers.Geometry.LineString( [
					new OpenLayers.Geometry.Point (bounds.left,  yl),
					new OpenLayers.Geometry.Point (bounds.right, yr)
				]));

				v2.style=this.gridStyle((y + 50000000) % 10000000, true);

				features.push (v2);
			}

			//------------------------------------------------------
			//	Add grid lines to vector layer
			//------------------------------------------------------

			this.addFeatures(features);

			//------------------------------------------------------
			//	Display grid size
			//------------------------------------------------------

			if (this.gridSizeDiv) {

				var meridian = (this.zone+60)%120*3-180;

				this.gridSizeDiv.innerHTML = OpenLayers.String.format(this.gridSizeText, {
					grid:	this.formatGridSize(unit),
					zone:	this.zone,
					left:	this.wrapLon(meridian - 1.5),
					center:	meridian,
					right:	this.wrapLon(meridian + 1.5)
				});

				this.gridSizeDiv.style.display=null;
			}
		}

		//--------------------------------------------------------------
		//	Superclass
		//--------------------------------------------------------------

		OpenLayers.Layer.Vector.prototype.moveTo.apply(this,arguments);
	},

	wrapLon: function (lon) {

		lon += 180;
		lon -= 360 * Math.floor (lon/360);
		lon -= 180;
		return lon;
	},

	eval: function(object, param) {

		return typeof(object)=='function'? object(param) : object;
	},

	gridStyle: function (value, isHorizontal) {

		value = value.toFixed(0);	// make integer

		return {
			labelAlign:      this.eval (this.gridLabelAlign, isHorizontal),
			label:           this.eval (this.gridLabel,      value),
			fontColor:       this.eval (this.gridLabelColor, value),
			fontSize:        this.eval (this.gridLabelSize,  value),
			strokeColor:     this.eval (this.gridColor,      value),
			strokeWidth:     this.eval (this.gridWidth,      value),
			strokeDashstyle: this.eval (this.gridDashStyle,  value),
			strokeOpacity:   this.eval (this.gridOpacity,    value)
		};
	},

	//----------------------------------------------------------------------
	//	Determine GK-Zone from WGS84-Longitude
	//----------------------------------------------------------------------

	coarseLonToZone: function (lon) {

		return Math.floor (lon/3.0+119.5) % 120 + 1;
	},

	//----------------------------------------------------------------------
	//	Main transformation WGS84 to GK
	//----------------------------------------------------------------------

	lonLatToGK: function (lonLat) {

		//--------------------------------------------------------------
		//	Sanity check and normalize
		//--------------------------------------------------------------

		if (lonLat.lat > 80.0) lonLat.lat = 80.0;
		if (lonLat.lat <-80.0) lonLat.lat =-80.0;

		lonLat.lon -= Math.floor(lonLat.lon/360+0.5)*360;

		//--------------------------------------------------------------
		//	Datum transformation
		//--------------------------------------------------------------

		lonLat = this.toBessel (lonLat);

		//--------------------------------------------------------------
		//	Zone and central meridian
		//--------------------------------------------------------------

		var zone = this.zone;
		var meridian = (zone+60)%120*3-180;

		//--------------------------------------------------------------
		//	Longitude to distance to central meridian
		//	Degrees to radian
		//--------------------------------------------------------------

		var phi = lonLat.lat/this.RHO;
		var lbd = (lonLat.lon-meridian - Math.floor ((lonLat.lon-meridian)/360.0+0.5)*360) / this.RHO;

		//--------------------------------------------------------------
		//	Aux values (Bessel-Ellipsoide)
		//--------------------------------------------------------------

		var sin = Math.sin (phi);
		var cos = Math.cos (phi);
		var tan = Math.tan (phi);
		var c2  = cos*cos;
		var t2  = tan*tan;
		var l2  = lbd*lbd;
		var eta = 0.00671921879917476 * c2;
		var Vq  = 1 + eta;
		var V   = Math.sqrt(Vq);
		var N   = 6399593.62575849/V;

		//--------------------------------------------------------------
		//	Conform transformation to Northing and Easting
		//	.oO( you are not expected to understand this :-)
		//--------------------------------------------------------------

		var nv = 6366742.52023404 * phi + sin * cos *
				(((0.00398709744545801 * c2 - 0.703273544958366) *
				c2 + 134.570531269045) * c2 - 32044.3278401524) +
				N * (0.5 * tan * c2 * l2 * (1 + 1/12 * (5+9*eta-t2)*c2*l2));
		var ev = N * cos * lbd * (1+ 1/6 * c2 * l2 * (Vq - t2 +0.05 * (5-18*t2+t2*t2) * c2 * l2));

		//--------------------------------------------------------------
		//	Return value
		//--------------------------------------------------------------

		var fzone = zone<10? '  '+zone : zone<100? ' '+zone : ''+zone;
		var fev   = Math.round (ev +   500000);
		var fnv   = Math.round (nv + 50000000).toString().substr(1);

		return {
			zone:	zone,
			x:	ev,
			y:	nv,
			fzone:	fzone,
			fx:	fev,
			fy:	fnv,
			text:	'GK ' + fzone + fev + ' ' + fnv
		};
	},

	//----------------------------------------------------------------------
	//	transform coordinates from WGS64 do Bessel
	//----------------------------------------------------------------------

	toBessel: function (lonLat) {

		//--------------------------------------------------------------
		//	Transform WGS84 ellipsoide to carthesian
		//--------------------------------------------------------------

		var phi_w = lonLat.lat / this.RHO;
		var lbd_w = lonLat.lon / this.RHO;

		var N_w = 6378137 / Math.sqrt (1 - 0.00669437998863492 * this.sqr(Math.sin(phi_w)));

		var w1 = N_w * Math.cos (phi_w) * Math.cos (lbd_w);
		var w2 = N_w * Math.cos (phi_w) * Math.sin (lbd_w);
		var w3 = N_w * Math.sin (phi_w) * 0.993305620011365;

		//--------------------------------------------------------------
		//	Transform ellipsoide (Helmert tranformation)
		//--------------------------------------------------------------

		var b1 = +0.999990181       * w1 -7.06993057967e-06 * w2 +3.56996494617e-07 * w3 -591.28;
		var b2 = +7.06993057967e-06 * w1 +0.999990181       * w2 +7.15992969596e-06 * w3 - 81.35;
		var b3 = -3.56996494617e-07 * w1 -7.15992969596e-06 * w2 +0.999990181       * w3 -396.39;

		//--------------------------------------------------------------
		//	Transform carthesian to elliposide (BESSEL)
		//--------------------------------------------------------------

		var p = Math.sqrt (b1*b1 + b2*b2);
		var theta = Math.atan2 (1.00335398479226 * b3, p);
		var lat = this.RHO * Math.atan2 (b3 + 42707.8852523705 * this.cub(Math.sin(theta)),
			p - 42565.1224788954 * this.cub(Math.cos(theta)));
		var lon = this.RHO * Math.atan2 (b2, b1);

		return {lon:lon, lat:lat};
	},

	//----------------------------------------------------------------------
	//	auxiliary constants, methods and vars
	//----------------------------------------------------------------------

	RHO: 180.0/3.14159265358979,

	sqr: function (x) { return x*x;  },
	cub: function (x) { return x*x*x;},

	zone: null,

	CLASS_NAME: "OpenLayers.Layer.GridGK"
});

//------------------------------------------------------------------------------
//	$Id: grid_gk.js,v 1.7 2019/06/25 13:35:28 wolf Exp $
//------------------------------------------------------------------------------
