//------------------------------------------------------------------------------
//	$Id: mouseposition_elevation.js,v 1.5 2019/07/22 23:46:35 wolf Exp $
//------------------------------------------------------------------------------
//	Erklaerung:	https://www.netzwolf.info/ol2/elevation.html
//------------------------------------------------------------------------------
//	Fragen, Wuensche, Bedenken, Anregungen?
//	<openlayers(%40)netzwolf.info>
//------------------------------------------------------------------------------

/*jsl:option explicit*/

'use strict';

OpenLayers.Control.MousePositionElevation = OpenLayers.Class (OpenLayers.Control, {

	url: null,

	//----------------------------------------------------------------------
	//	config
	//----------------------------------------------------------------------

	autoActivate:	true,
	prefixText:	'',
	suffixText:	'm',
	unknownText:	'',
	errorText:	'',

	//----------------------------------------------------------------------
	//	vars
	//----------------------------------------------------------------------

	div:		null,
	lastLonLat:	null,
	busy:		false,

	projection: new OpenLayers.Projection('EPSG:4326'),

	//----------------------------------------------------------------------
	//	control methods
	//----------------------------------------------------------------------

	destroy: function() {

		this.deactivate();
		OpenLayers.Control.prototype.destroy.apply(this, arguments);
	},

	draw: function() {

		OpenLayers.Control.prototype.draw.apply(this, arguments);
		return this.div;
	},

	activate: function() {

		if (!this.url) return false;

		if (OpenLayers.Control.prototype.activate.apply(this, arguments)) {

			this.map.events.register('mousemove', this, this.onMouseMove);
			this.div.innerHTML = this.unknownText;
			this.div.style.display = '';
			return true;
		} else {
			return false;
		}
	},

	deactivate: function() {

		if (OpenLayers.Control.prototype.deactivate.apply(this, arguments)) {

			this.map.events.unregister('mousemove', this, this.onMouseMove);
			this.div.style.display = 'none';
			this.div.innerHTML = this.unknownText;
			return true;
		} else {
			return false;
		}
	},

	onMouseMove: function(evt) {

		this.lastLonLat = this.map.getLonLatFromPixel(evt.xy);
		this.requestEle();
	},

	requestEle: function() {

		if (this.busy || !this.lastLonLat) return;

		var lonLat = this.lastLonLat; this.lastLonLat = null;
		lonLat.transform(this.map.getProjectionObject(), this.projection);

		OpenLayers.Request.GET({

			async:	true,
			url:	this.url + '?' + lonLat.lon + ',' + lonLat.lat,

			success: function(request) {

				this.busy = false;
				this.requestEle();

				var ele = null;

				try {
					ele = parseFloat(
						JSON.parse(request.responseText).firstEle);
				} catch(e) {

					this.div.innerHTML = '?';
					return;
				}

				if (!isNaN(ele)) {

					this.div.innerHTML = this.prefixText + ele + this.suffixText;

				} else {

					this.div.innerHTML = this.unknownText;
				}
			},

			failure: function(request) {

				this.div.innerHTML = this.errorText;
				var control = this;

				window.setTimeout(function() {

					control.busy = false;
					control.requestEle();
				}, 10000);
			},

			scope: this
		});

		this.busy = true;
	},

	CLASS_NAME: 'OpenLayers.Control.MousePositionElevation'
});

//------------------------------------------------------------------------------
//	$Id: mouseposition_elevation.js,v 1.5 2019/07/22 23:46:35 wolf Exp $
//------------------------------------------------------------------------------
